# default environment configs
default:
  name: {{ name }}

  # tao run path
  run_dir: {{ run_dir }}

  # It's highly recommended to add the path related options
  dataset_root:
  log_root:

  # If you want to enable functions related with Kaggle
  # (e.g. repo.sync_code_to_kaggle), config the following block.
  kaggle_username:
  kaggle_key:
  kaggle_code_dataset_id:

  # wandb related option
  wandb_secret:


# Add any other environments as you want, set TAO_ENV to switch
# between environments. The option in other environments will inherit
# from default environment
#
colab:

  # override default `dataset_root` option
  dataset_root:
